<?php

namespace s9e\TextFormatter\Tests\Configurator\Validators;

use s9e\TextFormatter\Tests\Test;
use s9e\TextFormatter\Configurator\Validators\AttributeName;

/**
* @covers s9e\TextFormatter\Configurator\Validators\AttributeName
*/
class AttributeNameTest extends Test
{
	/**
	* @testdox normalize() throws an InvalidArgumentException if the name is invalid
	* @expectedException InvalidArgumentException
	* @expectedExceptionMessage Invalid attribute name '*invalid*'
	*/
	public function testInvalid()
	{
		AttributeName::normalize('*invalid*');
	}

	// Start of content generated by ../scripts/patchValidatorsTests.php
	/**
	* @testdox "url" is normalized to "url"
	*/
	public function testValidF47645AE()
	{
		$this->assertTrue(AttributeName::isValid("url"));
		$this->assertSame("url", AttributeName::normalize("url"));
	}

	/**
	* @testdox "URL" is normalized to "url"
	*/
	public function testValid62D10724()
	{
		$this->assertTrue(AttributeName::isValid("URL"));
		$this->assertSame("url", AttributeName::normalize("URL"));
	}

	/**
	* @testdox "_url" is normalized to "_url"
	*/
	public function testValidB91D5C54()
	{
		$this->assertTrue(AttributeName::isValid("_url"));
		$this->assertSame("_url", AttributeName::normalize("_url"));
	}

	/**
	* @testdox "md5" is normalized to "md5"
	*/
	public function testValidE86CEBE1()
	{
		$this->assertTrue(AttributeName::isValid("md5"));
		$this->assertSame("md5", AttributeName::normalize("md5"));
	}

	/**
	* @testdox "5md" is invalid (name must start with a letter or an underscore)
	*/
	public function testInvalid476D4EC2()
	{
		$this->assertFalse(AttributeName::isValid("5md"));
	}

	/**
	* @testdox "" is invalid (name must start with a letter or an underscore)
	*/
	public function testInvalid0()
	{
		$this->assertFalse(AttributeName::isValid(""));
	}

	/**
	* @testdox "xmlns" is invalid ('xmlns' is reserved)
	*/
	public function testInvalidB5732726()
	{
		$this->assertFalse(AttributeName::isValid("xmlns"));
	}

	/**
	* @testdox "data-src" is normalized to "data-src"
	*/
	public function testValidE5FEAD2F()
	{
		$this->assertTrue(AttributeName::isValid("data-src"));
		$this->assertSame("data-src", AttributeName::normalize("data-src"));
	}

	/**
	* @testdox "foo:bar" is invalid (no colons allowed)
	*/
	public function testInvalid81E2B6CF()
	{
		$this->assertFalse(AttributeName::isValid("foo:bar"));
	}

	/**
	* @testdox "foo\n" is invalid (no newlines allowed)
	*/
	public function testInvalid7E3265A8()
	{
		$this->assertFalse(AttributeName::isValid("foo\n"));
	}
	// End of content generated by ../scripts/patchValidatorsTests.php

	public static function getNames()
	{
		return [
			'url'      => 'url',
			'URL'      => 'url',
			'_url'     => '_url',
			'md5'      => 'md5',
			'5md'      => 'Invalid: name must start with a letter or an underscore',
			''         => 'Invalid: name must start with a letter or an underscore',
			'xmlns'    => "Invalid: 'xmlns' is reserved",
			'data-src' => 'data-src',
			'foo:bar'  => 'Invalid: no colons allowed',
			"foo\n"    => 'Invalid: no newlines allowed'
		];
	}
}